#ifndef HEXMAP_H
#define HEXMAP_H

#include "..\Tiles\Tile.h"

// Nomes dos ambientes
static string ambientes[96];
// Caracteres de representacao
static char caracteres[96];
static bool initialized = 0;

// Classe que implementa o mapa de hexagonos
// Mapa Hexagonal com coordenadas axiais/trapezoidais
// referencia aa dimensao r.
class HexMap {
    public:
        
        HexMap(int size);
        // Inicializa os vetores de caracteres e ambientes
        static void initialize(string file);
        // Retorna caracteres[index]
        static char getChar (int index);
        // Insere "val" na posicao [r,q]
        void insertTile(Tile* val, int r, int q);
        
        // Retorna o ambiente na posicao [r,q]
        string getAmb(int r, int q);
        // Retorna ambientes[index]
        string getAmb(int index);

        // Retorna a Tile na posicao [r,q]
        Tile* getTile(int r, int q);
        // Imprime o mapa
        void printMap();
        // Alterna o atributo "revelado" na Tile em [r,q]
        bool reveal(int r, int q);
        // Alterna o atributo "marcado" na Tile em [r,q]
        bool mark(int r, int q);

        // Ferramentas de debug por enquanto:

        // "Desenha" uma linha partindo de [rS,qS] na direcao
        void drawLine (int dir, int rS, int qS);
        // NAO IMPLEMENTADO 
        // "Desenha" uma linha entre [rS,qS] e [rF,qF] 
        void drawLine (int rStart, int qStart, int rFinish, int qFinish);

        // Mostra as Tiles adjacentes aa Tile em [r,q]
        void pingAdj (int r, int q);

        // Converte coordenadas [r,q] para indices [i,j] na matriz "mapa"
        int* coordToIndex(int r, int q);

        // Checa se a Tile em [r,q] eh uma Tile valida
        bool inBounds(int r, int q);
        ~HexMap();
    private: 
        // atualiza as adjacencias da tile r,q
        void setAdjacencias(int r, int q);
        
        Tile*** mapa;
        // Garante que mapa inicia apontando para Tile* = NULL
        void zeraVetores();     

        int size;

        
};
#endif